; Black Gold rel 0.02
; Created by Graechan
; Adapted from tutorial by Conbagui at http://www.gimpchat.com/viewtopic.php?f=23&t=8895
; Comments directed to http://gimpchat.com or http://gimpscripts.com
;
; License: GPLv3
;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;    GNU General Public License for more details.
;
;    To view a copy of the GNU General Public License
;    visit: http://www.gnu.org/licenses/gpl.html
;
;
; ------------
;| Change Log |
; ------------ 
; Rel 0.01 - Initial Release
; Rel 0.02 - Changed the default background to one that is compatible to 2.6 and 2.8
;
(define list-blend-dir '("Left to Right" "Top to Bottom" "Diagonal"))
;
; include layer Procedure
(define (include-layer image newlayer oldlayer stack)	;stack 0=above 1=below
	(cond ((defined? 'gimp-image-get-item-position) ;test for 2.8 compatability
            (gimp-image-insert-layer image newlayer (car (gimp-item-get-parent oldlayer)) 
			(+ (car (gimp-image-get-layer-position image oldlayer)) stack))                                     ;For GIMP 2.8 
          )
          (else
           (gimp-image-add-layer image newlayer (+ (car (gimp-image-get-layer-position image oldlayer)) stack)) ;For GIMP 2.6 
          )
    ) ;end cond
) ;end add layer procedure
;
;find layer by name proceedure
(define (find-layer-by-name image layerName)
  (let* (
           (layerList (vector->list (cadr (gimp-image-get-layers image))))    
           (wantedLayerId -1)
		   (layerId 0)
           (layerText "")
        )
       
        (while (not (null? layerList))
          (set! layerId (car layerList))
		  (set! layerText (cond ((defined? 'gimp-image-get-item-position) (car (gimp-item-get-name layerId)))
		                  (else (car (gimp-drawable-get-name layerId)))))
          (if (string=? layerText layerName) (set! wantedLayerId layerId))          
          (set! layerList (cdr layerList))) ;endwhile        
        (if (= -1 wantedLayerId) (error (string-append "Could not find a layer with name:- " layerName)))
        (list wantedLayerId)
  ) ;end variables
) ;end find layer by name proceedure
;
(define (script-fu-black-gold 
                                      text
									  letter-spacing
									  line-spacing
                                      font-in 
                                      font-size
									  outline-gradient
                                      bkg-type 
                                      pattern
                                      bkg-color
									  gradient
									  gradient-type
									  reverse
									  blendir
									  conserve
									  )
  (let* (
         (image (car (gimp-image-new 256 256 RGB)))         
         (border (/ font-size 4))
		 (font (if (> (string-length font-in) 0) font-in (car (gimp-context-get-font))))
         (size-layer (car (gimp-text-fontname image -1 0 0 text border TRUE font-size PIXELS font)))
         (final-width (car (gimp-drawable-width size-layer)))
         (final-height (car (gimp-drawable-height size-layer)))
         (text-layer 0)
		 (cent-layer 0)
         (width 0)
         (height 0)
         (bkg-layer 0)
		 (ver 2.8)
		 (selection-channel 0)
		 (cent-width 0)
		 (cent-height 0)
         (offx 0)
		 (offy 0)
		 (aspect 0)
		 (innerglow 0)
		 (3d-layer 0)
		 (copy-layer 0)
		 (drop-shadow 0)
		 (x1 0)
		 (y1 0)
		 (x2 0)
		 (y2 0)
		 (H1 12)
		 (H2 12)
		 (V1 12)
		 (V2 12)
		 (V3 12)
		 (V4 12)
		 (Object_Name "Black-Gold")
         )
	(cond ((not (defined? 'gimp-image-get-item-position)) (set! ver 2.6))) ;define the gimp version	 
    
	(gimp-context-push)
	(gimp-context-set-paint-method "gimp-paintbrush")
	(if (= ver 2.8) (gimp-context-set-dynamics "Dynamics Off"))
	(gimp-context-set-foreground '(0 0 0))
	(gimp-context-set-background '(255 255 255))

;;;;Add the text layer for a temporary larger Image size
    (set! text-layer (car (gimp-text-fontname image -1 0 0 text (round (/ 450 4)) TRUE 450 PIXELS font)))
	(gimp-drawable-set-name text-layer "Text")
;;;;adjust text 
	(gimp-text-layer-set-justification text-layer 2)
	(gimp-text-layer-set-letter-spacing text-layer letter-spacing)
	(gimp-text-layer-set-line-spacing text-layer line-spacing)
;;;;set the new width and height	
    (set! width (car (gimp-drawable-width text-layer)))
    (set! height (car (gimp-drawable-height text-layer)))    
    (gimp-image-remove-layer image size-layer)
    (gimp-image-resize-to-layers image)
	
    (if (< line-spacing 0) (begin
	(gimp-image-crop image width (+ height line-spacing) 0 (-(/ line-spacing 2)))
	(set! width (car (gimp-drawable-width text-layer)))
    (set! height (car (gimp-drawable-height text-layer)))
	)) ;endif

;;;;square up the Image if needed	
    ;(if (< height width) (set! height width))
	;(if (< width height) (set! width height))

;;;;set the text clolor    
    (cond ((= ver 2.8) (gimp-image-select-item image 2 text-layer)) 
	(else (gimp-selection-layer-alpha text-layer)))
	(gimp-edit-fill text-layer FOREGROUND-FILL)
	
;;;;create selection-channel (gimp-selection-load selection-channel)
    (set! selection-channel (car (gimp-selection-save image)))	
	(gimp-drawable-set-name selection-channel "selection-channel")
    (gimp-image-set-active-layer image text-layer)	
	(gimp-selection-none image)
    
;;;;begin the script
    (black-gold-layerfx-inner-glow image
				      text-layer ;drawable
				      '(255 255 190) ;color
				      75 ;opacity
				      8 ;contour
				      0 ;noise
				      4 ;mode
				      0 ;source
				      4 ;choke
				      12 ;size
				      FALSE) ;merge	
	(set! innerglow (car (gimp-image-get-active-layer image)))
	
;;;;select the innerglow layer and blend with "Golden" gradient
    (cond ((= ver 2.8) (gimp-image-select-item image 2 innerglow)) 
	(else (gimp-selection-layer-alpha innerglow)))	
	(gimp-context-set-gradient outline-gradient)
	(gimp-edit-blend innerglow CUSTOM-MODE NORMAL-MODE GRADIENT-LINEAR 100 0 REPEAT-NONE FALSE FALSE 3 0.2 TRUE	0 (-(/ height 2)) 0 height)
	(gimp-selection-none image)
	(set! text-layer (car (gimp-image-merge-down image innerglow EXPAND-AS-NECESSARY)))

;;;;apply a bevel to the Text-layer	
	(plug-in-gauss-rle2 RUN-NONINTERACTIVE image text-layer 3 3)
	(black-gold-layerfx-bevel-emboss image
					text-layer ;drawable
					1 ;style
					40 ;depth
					0 ;direction
					21 ;size
					14 ;soften
					108 ;angle
					30 ;altitude
					3 ;glosscontour
					'(255 255 255) ;highlightcolor
					4 ;highlightmode
					75 ;highlightopacity
					'(0 0 0) ;shadowcolor
					2 ;shadowmode
					16.7 ;shadowopacity
					10 ;surfacecontour
					FALSE ;invert
					TRUE) ;merge
	(set! text-layer (car (gimp-image-get-active-layer image)))
	
;;;;apply map-object to the text-layer	
	(plug-in-map-object RUN-NONINTERACTIVE image text-layer 
			0			         ; Type of mapping (0=plane,1=sphere,2=box,3=cylinder)
			0.5 0.5 2.0		     ; viewpoint
			0.5 0.5 0.0		     ; object pos
			1.0 0.0 0.0		     ; first axis
			0.0 1.0 0.0		     ; 2nd axis
			0.0 20 0.0	         ; axis rotation
			1 '(255 255 255)     ; Type of lightsource (0=point,1=directional,2=none)and Lightsource color (r,g,b) 
			-0.5 -0.5 2		     ; light position
			-1.0 -1.0 1.0	     ; light direction
			0.3 1.2 0.7 0.0 27.0 ; material (amb, diff, refl, spec, high)
			TRUE		         ; antialias
			FALSE 		         ; tile
			FALSE 		         ; new image
			TRUE 		         ; transparency
			.5		             ; Sphere/cylinder radius (only used when maptype=1 or 3)
			0.5 0.5 0.5    	     ; box size
			1.00                 ; unused parameter Cylinder length
			-1                   ; Box front face (set these to -1 if not used)
			-1                   ; Box back face
			-1                   ; Box top face 
			-1                   ; Box bottom face
			-1                   ; Box left face
			-1                   ; Box right face
			-1                   ; Cylinder top face (unused parameter)
			-1)                  ; Cylinder bottom face (unused parameter)
	
	
	(black-gold-make-3d image text-layer 
	                          30 ;Depth 
							  2 ;Horizontal direction 
							  1 ;Vertical direction 
							  0 ;Half depth 
							  45 ;Bump direction 
							  10 ;Bump height 
							  2) ;Bump-projection depth
	
;;;;create the 3d-layer copy
	(gimp-drawable-set-visible text-layer FALSE)
	(set! 3d-layer (car (gimp-image-get-active-layer image)))
	(gimp-layer-resize-to-image-size 3d-layer)
	(set! copy-layer (car (gimp-layer-copy 3d-layer TRUE)))
	(include-layer image copy-layer 3d-layer 0)	;stack 0=above 1=below
	(gimp-layer-set-mode copy-layer DODGE-MODE) 
	
;;;;bevel the 3d-layer copy
	(black-gold-layerfx-bevel-emboss image
					copy-layer ;drawable
					1 ;style
					40 ;depth
					0 ;direction
					21 ;size
					16 ;soften
					148 ;angle
					30 ;altitude
					3 ;glosscontour
					'(255 255 255) ;highlightcolor
					4 ;highlightmode
					60 ;highlightopacity
					'(0 0 0) ;shadowcolor
					11 ;shadowmode
					16.7 ;shadowopacity
					10 ;surfacecontour
					FALSE ;invert
					TRUE) ;merge
	(set! copy-layer (car (gimp-image-get-active-layer image)))
	(gimp-layer-set-mode copy-layer DODGE-MODE)
	
;;;;Scale Image to it's final size;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (set! aspect (/ final-width (car (gimp-image-width image)))) 
    (gimp-image-scale image final-width (* (car (gimp-image-height image)) aspect))
	(set! width (car (gimp-image-width image)))
	(set! height (car (gimp-image-height image)))    
	  
;;;;create the background layer    
	
	(set! bkg-layer (car (gimp-layer-new image width height RGBA-IMAGE "Background" 100 NORMAL-MODE)))
	(include-layer image bkg-layer 3d-layer 1)	;stack 0=above 1=below
    
	(gimp-context-set-pattern pattern)
	(gimp-context-set-background bkg-color)
	(gimp-context-set-gradient gradient)
	(if (= bkg-type 2) (gimp-drawable-fill bkg-layer PATTERN-FILL))		
    (if (= bkg-type 1) (gimp-drawable-fill bkg-layer BACKGROUND-FILL))	
    (if (= bkg-type 3) 
	(begin
	(gimp-selection-none image)
	(gimp-drawable-fill bkg-layer BACKGROUND-FILL)
    (if (= blendir 0) (set! x2 width))
	(if (= blendir 1) (set! y2 height))
	(if (= blendir 2) (begin
	(set! x2 (/ width 2))
	(set! y2 (/ height 2))))	
	(gimp-edit-blend bkg-layer CUSTOM-MODE NORMAL-MODE gradient-type 100 0 REPEAT-NONE reverse FALSE 3 0.2 TRUE x1 y1 x2 y2)))
    (if (= bkg-type 0) 
	(begin
	(black-gold-metal-mesh image 
                               bkg-layer ;layer
                               0 ;type
							   64 ;size
							   FALSE ;keep-selection-in
							   FALSE) ;conserve
	)) ;endif	
    ;END

	
	(if (= conserve FALSE)	(begin
	(gimp-image-remove-layer image text-layer)
	(set! text-layer (car (gimp-image-merge-visible-layers image EXPAND-AS-NECESSARY)))
	(gimp-drawable-set-name text-layer "Black Gold")))
	
	(gimp-context-pop)	
    (gimp-display-new image)
	
    )
  )
  
(script-fu-register "script-fu-black-gold"
  "Black Gold"
  "Create a Logo and a Background"
  "Graechan"
  "Graechan - http://gimpchat.com"
  "June 2011"
  ""
  SF-TEXT       "Text"    "Black\nand\nGold"
  SF-ADJUSTMENT "Letter Spacing" '(0 -100 100 1 5 0 0)
  SF-ADJUSTMENT "Line Spacing" '(-200 -8192 8292 1 5 0 0)
  SF-FONT       "Font"           "Sans Bold" ;"Underground NF"
  SF-ADJUSTMENT "Font size (pixels)" '(250 6 500 1 1 0 1)
  SF-GRADIENT   "Outline Gradient" "Golden"
  SF-OPTION "Background Type" '("Default" "Color" "Pattern" "Gradient" "Transparent")
  SF-PATTERN    "Pattern"            "Pink Marble"
  SF-COLOR      "Background color"         "Blue"
  SF-GRADIENT   "Background Gradient" "Abstract 3"
  SF-ENUM "Gradient Fill Mode" '("GradientType" "gradient-linear")
  SF-TOGGLE     "Reverse the Gradient"   FALSE
  SF-OPTION		"Blend Direction" 		list-blend-dir
  SF-TOGGLE     "Keep the Layers"   FALSE
  )
(script-fu-menu-register "script-fu-black-gold" "<Image>/File/Create/Logos")

(define (get-blending-mode mode)
  (let* ((modenumbers #(0 1 3 15 4 5 16 17 18 19 20 21 6 7 8 9 10 11 12 13 14)))
    (vector-ref modenumbers mode)
  )
)

(define (math-round input)
  (floor (+ input 0.5))
)

(define (math-ceil input)
  (if (= input (floor input))
    input
    (+ (floor input) 1)
  )
)

(define (get-layer-pos img layer)
  (let* ((layerdata (gimp-image-get-layers img))
	 (numlayers (car layerdata))
	 (layerarray (cadr layerdata))
	 (i 0)
	 (pos -1)
	)
    (while (< i numlayers)
      (if (= layer (vector-ref layerarray i))
	(begin
	  (set! pos i)
	  (set! i numlayers)
	)
	(set! i (+ i 1))
      )
    )
    pos
  )
)

(define (add-under-layer img newlayer oldlayer)
  (gimp-image-add-layer img newlayer (+ (get-layer-pos img oldlayer) 1))
)

(define (add-over-layer img newlayer oldlayer)
  (gimp-image-add-layer img newlayer (get-layer-pos img oldlayer))
)

(define (draw-blurshape img drawable size initgrowth sel invert)
  (let* ((k initgrowth)
	 (currshade 0)
	 (i 0))
    (while (< i size)
      (if (> k 0)
	(gimp-selection-grow img k)
	(if (< k 0)
	  (gimp-selection-shrink img (abs k))
	)
      )
      (if (= invert 1)
	(set! currshade (math-round (* (/ (- size (+ i 1)) size) 255)))
	(set! currshade (math-round (* (/ (+ i 1) size) 255)))
      )
      (gimp-palette-set-foreground (list currshade currshade currshade))
      (if (= (car (gimp-selection-is-empty img)) 0)
	(gimp-edit-fill drawable 0)
      )
      (gimp-selection-load sel)
      (set! k (- k 1))
      (set! i (+ i 1))
    )
  )
)

(define (apply-contour drawable channel contour)
  (let* ((contourtypes #(0 0 0 0 0 0 0 0 0 1 1))
	 (contourlengths #(6 6 10 14 18 10 18 18 10 256 256))
	 (contours #(#(0 0 127 255 255 0)
#(0 255 127 0 255 255)
#(0 64 94 74 150 115 179 179 191 255)
#(0 0 5 125 6 125 48 148 79 179 107 217 130 255)
#(0 0 33 8 64 38 97 102 128 166 158 209 191 235 222 247 255 255)
#(0 0 28 71 87 166 194 240 255 255)
#(0 0 33 110 64 237 97 240 128 138 158 33 191 5 222 99 255 255)
#(0 0 33 74 64 219 97 186 128 0 158 176 191 201 222 3 255 255)
#(3 255 54 99 97 107 179 153 252 0)
#(0 5 9 13 16 19 22 25 27 29 30 32 33 34 35 36 38 39 40 41 43 44 46 47 48 49 50 51 52 53 54 55 55 56 56 57 57 58 58 59 59 59 60 60 60 61 61 61 61 62 62 62 62 62 63 63 63 63 63 63 64 64 64 64 64 71 75 78 81 84 86 89 91 93 95 96 98 99 101 102 103 104 105 107 107 108 110 111 112 113 114 115 116 117 118 119 119 120 121 121 122 123 123 123 124 124 124 125 125 125 125 125 125 125 126 126 126 126 126 126 126 125 125 125 125 125 125 125 125 130 134 137 141 145 148 151 153 156 158 160 162 163 165 166 167 168 170 171 171 172 173 174 175 176 177 178 178 179 180 181 181 182 183 183 184 184 185 185 186 186 187 187 188 188 189 189 189 189 190 190 190 190 191 191 191 191 191 191 191 191 191 191 193 194 196 197 198 200 201 203 204 205 207 208 209 211 212 213 214 215 217 218 219 220 220 221 222 222 223 223 224 224 224 224 224 223 223 222 222 221 221 220 219 218 217 216 215 214 213 212 211 210 209 208 206 205 204 203 202 200 199 198 197 196 194 194)
#(0 2 4 6 8 10 12 14 16 18 20 22 24 26 28 30 32 34 36 38 40 42 44 46 48 50 52 54 56 58 60 62 64 66 68 70 72 74 76 78 80 82 84 86 88 90 92 94 96 98 100 102 104 106 108 110 112 114 116 118 120 122 124 126 127 125 123 121 119 117 115 113 111 109 107 105 103 101 99 97 95 93 91 89 87 85 83 81 79 77 75 73 71 69 67 65 63 61 59 57 55 53 51 49 47 45 43 41 39 37 35 33 31 29 27 25 23 21 19 17 15 13 11 9 7 5 3 1 1 3 5 7 9 11 13 15 17 19 21 23 25 27 29 31 33 35 37 39 41 43 45 47 49 51 53 55 57 59 61 63 65 67 69 71 73 75 77 79 81 83 85 87 89 91 93 95 97 99 101 103 105 107 109 111 113 115 117 119 121 123 125 127 128 126 124 122 120 118 116 114 112 110 108 106 104 102 100 98 96 94 92 90 88 86 84 82 80 78 76 74 72 70 68 66 64 62 60 58 56 54 52 50 48 46 44 42 40 38 36 34 32 30 28 26 24 22 20 18 16 14 12 10 8 6 4 2))))
    (if (= (vector-ref contourtypes (- contour 1)) 0)
      (gimp-curves-spline drawable channel (vector-ref contourlengths (- contour 1)) (vector-ref contours (- contour 1)))
      (gimp-curves-explicit drawable channel (vector-ref contourlengths (- contour 1)) (vector-ref contours (- contour 1)))
    )
  )
)

(define (black-gold-layerfx-inner-glow img
				      drawable
				      color
				      opacity
				      contour
				      noise
				      mode
				      source
				      choke
				      size
				      merge)
  (gimp-image-undo-group-start img)
  (let* ((origfgcolor (car (gimp-palette-get-foreground)))
	 (origselection (car (gimp-selection-save img)))
	 (drwwidth (car (gimp-drawable-width drawable)))
	 (drwheight (car (gimp-drawable-height drawable)))
	 (layername (car (gimp-drawable-get-name drawable)))
	 (drwoffsets (gimp-drawable-offsets drawable))
	 (glowlayer (car (gimp-layer-new img drwwidth drwheight (cond ((= (car (gimp-image-base-type img)) 0) 1) ((= (car (gimp-image-base-type img)) 1) 3)) (string-append layername "-innerglow") opacity (get-blending-mode mode))))
	 (glowmask 0)
	 (alphaSel 0)
	 (shrinkamt (* (/ choke 100) size))
	 (steps (- size shrinkamt))
	 (i 0)
	 (currshade 0)
	 (origmask 0)
	 (alphamask 0)
	)
    (add-over-layer img glowlayer drawable)
    (gimp-layer-set-offsets glowlayer (car drwoffsets) (cadr drwoffsets))
    (gimp-selection-all img)
    (gimp-palette-set-foreground color)
    (gimp-edit-fill glowlayer 0)
    (gimp-selection-none img)
    (set! glowmask (car (gimp-layer-create-mask glowlayer 1)))
    (gimp-layer-add-mask glowlayer glowmask)
    (gimp-selection-layer-alpha drawable)
    (if (> (car (gimp-layer-get-mask drawable)) -1)
      (gimp-selection-combine (car (gimp-layer-get-mask drawable)) 3)
    )
    (set! alphaSel (car (gimp-selection-save img)))
    (if (= source 0)
      (begin
	(gimp-selection-all img)
	(gimp-palette-set-foreground '(255 255 255))
	(gimp-edit-fill glowmask 0)
	(gimp-selection-load alphaSel)
	(draw-blurshape img glowmask steps (- (* shrinkamt -1) 1) alphaSel 1)
      )
      (draw-blurshape img glowmask steps (* shrinkamt -1) alphaSel 0)
    )
    (gimp-selection-none img)
    (if (> contour 0)
      (apply-contour glowmask 0 contour)
    )
    (if (and (= source 0) (= merge 0))
      (begin
	(gimp-selection-load alphaSel)
	(gimp-selection-invert img)
	(gimp-palette-set-foreground '(0 0 0))
	(gimp-edit-fill glowmask 0)
      )
    )
    (if (> noise 0)
      (apply-noise img drawable glowlayer noise)
    )
    (gimp-layer-remove-mask glowlayer 0)
    (if (= merge 1)
      (if (= source 0)
	(begin
	  (set! origmask (car (gimp-layer-get-mask drawable)))
	  (if (> origmask -1)
	    (begin
	      (set! origmask (car (gimp-channel-copy origmask)))
	      (gimp-layer-remove-mask drawable 1)
	    )
	  )
	  (set! alphamask (car (gimp-layer-create-mask drawable 3)))
	  (set! glowlayer (car (gimp-image-merge-down img glowlayer 0)))
	  (gimp-drawable-set-name glowlayer layername)
	  (gimp-layer-add-mask glowlayer alphamask)
	  (gimp-layer-remove-mask glowlayer 0)
	  (if (> origmask -1)
	    (gimp-layer-add-mask glowlayer origmask)
	  )
	)
	(begin
	  (set! origmask (car (gimp-layer-get-mask drawable)))
	  (if (> origmask -1)
	    (begin
	      (set! origmask (car (gimp-channel-copy origmask)))
	      (gimp-layer-remove-mask drawable 1)
	    )
	  )
	  (set! glowlayer (car (gimp-image-merge-down img glowlayer 0)))
	  (gimp-drawable-set-name glowlayer layername)
	  (if (> origmask -1)
	    (gimp-layer-add-mask glowlayer origmask)
	  )
	)
      )
    )
    (gimp-selection-none img)
    (gimp-palette-set-foreground origfgcolor)
    (gimp-selection-load origselection)
    (gimp-image-remove-channel img alphaSel)
    (gimp-image-remove-channel img origselection)
	(gimp-image-set-active-layer img glowlayer)
	
    (gimp-displays-flush)
  )
  (gimp-image-undo-group-end img)
)

(define (black-gold-layerfx-bevel-emboss img
					drawable
					style
					depth
					direction
					size
					soften
					angle
					altitude
					glosscontour
					highlightcolor
					highlightmode
					highlightopacity
					shadowcolor
					shadowmode
					shadowopacity
					surfacecontour
					invert
					merge)
  (gimp-image-undo-group-start img)
  (let* ((origfgcolor (car (gimp-palette-get-foreground)))
	 (origselection (car (gimp-selection-save img)))
	 (drwwidth (car (gimp-drawable-width drawable)))
	 (drwheight (car (gimp-drawable-height drawable)))
	 (drwoffsets (gimp-drawable-offsets drawable))
	 (layername (car (gimp-drawable-get-name drawable)))
	 (imgtype (cond ((= (car (gimp-image-base-type img)) 0) 1) ((= (car (gimp-image-base-type img)) 1) 3)))
	 (lyrgrowamt (math-round (* size 1.2)))
	 (bumpmaplayer 0)
	 (highlightlayer 0)
	 (highlightmask 0)
	 (shadowlayer 0)
	 (shadowmask 0)
	 (layersize 0)
	 (alphaSel 0)
	 (halfsizef 0)
	 (halfsizec 0)
	 (origmask 0)
	 (alphamask 0)
	)
    (cond
      ((= style 0)
	(begin
	  (set! layersize (list
	    (+ drwwidth (* lyrgrowamt 2))
	    (+ drwheight (* lyrgrowamt 2))
	    (- (car drwoffsets) lyrgrowamt)
	    (- (cadr drwoffsets) lyrgrowamt)
	  ))
	)
      )
      ((= style 1)
	(begin
	  (set! layersize (list
	    drwwidth
	    drwheight
	    (car drwoffsets)
	    (cadr drwoffsets)
	  ))
	)
      )
      ((= style 2)
	(begin
	  (set! layersize (list
	    (+ drwwidth lyrgrowamt)
	    (+ drwheight lyrgrowamt)
	    (- (car drwoffsets) (floor (/ lyrgrowamt 2)))
	    (- (cadr drwoffsets) (floor (/ lyrgrowamt 2)))
	  ))
	)
      )
      (
	(begin
	  (set! layersize (list
	    (+ drwwidth lyrgrowamt)
	    (+ drwheight lyrgrowamt)
	    (- (car drwoffsets) (floor (/ lyrgrowamt 2)))
	    (- (cadr drwoffsets) (floor (/ lyrgrowamt 2)))
	  ))
	)
      )
    )
    (set! bumpmaplayer (car (gimp-layer-new img (car layersize) (cadr layersize) imgtype (string-append layername "-bumpmap") 100 0)))
    (set! highlightlayer (car (gimp-layer-new img (car layersize) (cadr layersize) imgtype (string-append layername "-highlight") highlightopacity (get-blending-mode highlightmode))))
    (set! shadowlayer (car (gimp-layer-new img (car layersize) (cadr layersize) imgtype (string-append layername "-shadow") shadowopacity (get-blending-mode shadowmode))))
    (add-over-layer img bumpmaplayer drawable)
    (add-over-layer img shadowlayer bumpmaplayer)
    (add-over-layer img highlightlayer shadowlayer)
    (gimp-layer-set-offsets bumpmaplayer (caddr layersize) (cadddr layersize))
    (gimp-layer-set-offsets shadowlayer (caddr layersize) (cadddr layersize))
    (gimp-layer-set-offsets highlightlayer (caddr layersize) (cadddr layersize))
    (gimp-selection-all img)
    (gimp-palette-set-foreground highlightcolor)
    (gimp-edit-fill highlightlayer 0)
    (gimp-palette-set-foreground shadowcolor)
    (gimp-edit-fill shadowlayer 0)
    (gimp-palette-set-foreground '(0 0 0))
    (gimp-edit-fill bumpmaplayer 0)
    (set! highlightmask (car (gimp-layer-create-mask highlightlayer 1)))
    (set! shadowmask (car (gimp-layer-create-mask shadowlayer 1)))
    (gimp-layer-add-mask highlightlayer highlightmask)
    (gimp-layer-add-mask shadowlayer shadowmask)
    (gimp-selection-layer-alpha drawable)
    (if (> (car (gimp-layer-get-mask drawable)) -1)
       (gimp-selection-combine (car (gimp-layer-get-mask drawable)) 3)
    )
    (set! alphaSel (car (gimp-selection-save img)))
    (cond
      ((= style 0)
	(draw-blurshape img bumpmaplayer size size alphaSel 0)
      )
      ((= style 1)
	(draw-blurshape img bumpmaplayer size 0 alphaSel 0)
      )
      ((= style 2)
	(begin
	  (set! halfsizec (math-ceil (/ size 2)))
	  (draw-blurshape img bumpmaplayer size halfsizec alphaSel 0)
	)
      )
      (
	(begin
	  (set! halfsizef (floor (/ size 2)))
	  (set! halfsizec (- size halfsizef))
	  (gimp-selection-all img)
	  (gimp-palette-set-foreground '(255 255 255))
	  (gimp-edit-fill bumpmaplayer 0)
	  (draw-blurshape img bumpmaplayer halfsizec halfsizec alphaSel 1)
	  (draw-blurshape img bumpmaplayer halfsizef 0 alphaSel 0)
	)
      )
    )
    (gimp-selection-all img)
    (gimp-palette-set-foreground '(127 127 127))
    (gimp-edit-fill highlightmask 0)
    (gimp-selection-none img)
    (if (> surfacecontour 0)
      (apply-contour bumpmaplayer 0 surfacecontour)
    )
    (if (< angle 0)
      (set! angle (+ angle 360))
    )
    (plug-in-bump-map 1 img highlightmask bumpmaplayer angle altitude depth 0 0 0 0 1 direction 0)
    (if (> glosscontour 0)
      (apply-contour highlightmask 0 glosscontour)
    )
    (if (> soften 0)
      (plug-in-gauss-rle 1 img highlightmask soften 1 1)
    )
    (if (> invert 0)
      (gimp-invert highlightmask)
    )
    (gimp-channel-combine-masks shadowmask highlightmask 2 0 0)
    (gimp-levels highlightmask 0 127 255 1.0 0 255)
    (gimp-levels shadowmask 0 0 127 1.0 255 0)
    (gimp-selection-load alphaSel)
    (if (= style 0)
      (gimp-selection-grow img size)
      (if (or (= style 2) (= style 3))
	(gimp-selection-grow img halfsizec)
      )
    )
    (gimp-selection-invert img)
    (gimp-palette-set-foreground '(0 0 0))
    (gimp-edit-fill shadowmask 0)
    (gimp-selection-none img)
    (gimp-image-remove-layer img bumpmaplayer)
    (if (= merge 1)
      (if (= style 1)
	(begin
	  (set! origmask (car (gimp-layer-get-mask drawable)))
	  (if (> origmask -1)
	    (begin
	      (set! origmask (car (gimp-channel-copy origmask)))
	      (gimp-layer-remove-mask drawable 1)
	    )
	  )
	  (set! alphamask (car (gimp-layer-create-mask drawable 3)))
	  (set! shadowlayer (car (gimp-image-merge-down img shadowlayer 0)))
	  (set! highlightlayer (car (gimp-image-merge-down img highlightlayer 0)))
	  (gimp-drawable-set-name highlightlayer layername)
	  (gimp-layer-add-mask highlightlayer alphamask)
	  (gimp-layer-remove-mask highlightlayer 0)
	  (if (> origmask -1)
	    (gimp-layer-add-mask highlightlayer origmask)
	  )
	)
	(begin
	  (set! origmask (car (gimp-layer-get-mask drawable)))
	  (if (> origmask -1)
	    (gimp-layer-remove-mask drawable 0)
	  )
	  (set! shadowlayer (car (gimp-image-merge-down img shadowlayer 0)))
	  (set! highlightlayer (car (gimp-image-merge-down img highlightlayer 0)))
	  (gimp-drawable-set-name highlightlayer layername)
	)
      )
    )
    (gimp-palette-set-foreground origfgcolor)
    (gimp-selection-load origselection)
    (gimp-image-remove-channel img alphaSel)
    (gimp-image-remove-channel img origselection)
    (gimp-displays-flush)
  )
  (gimp-image-undo-group-end img)
)

(define (black-gold-make-3d beeld laag diepte hrichting vrichting half brichting bhoogte bprojdiep)
		(gimp-image-undo-group-start beeld)
	(let*
		(
		   (orglaag (car (gimp-image-get-active-layer beeld)))
		)
	  (let*
		(
		   (werklaag (car (gimp-layer-copy laag TRUE )))
		   (pos (car (gimp-image-get-layer-position beeld laag)))
		   (nwpos (+  pos 1))
	  	)
	    (gimp-image-add-layer beeld werklaag nwpos)
	  )
	  (let* 
	    (
;	      (nlaag 0)
	      (baslaag (car (gimp-image-get-active-layer beeld)))
;		  (baspos (car (gimp-image-get-layer-position beeld baslaag)))
	      (lagenteller diepte)
		  (focuslaag baslaag)
		  (hwissel 0)
		  (vwissel 0)
	    )
		  (set! hrichting (- 1 hrichting))
		  (set! vrichting( - 1 vrichting))
		  (set! hwissel hrichting)
		  (set! vwissel vrichting)
	      (plug-in-bump-map 1 beeld baslaag baslaag brichting bhoogte bprojdiep 0 0 0 0 TRUE FALSE 0) 
		  (while (> lagenteller 0)
			  (let*
	    	    (
				  (focuslaag (car (gimp-image-get-active-layer beeld)))
			      (werklaag (car (gimp-layer-copy focuslaag TRUE )))
			      (laagpos (car (gimp-image-get-layer-position beeld focuslaag)))
				  (nwpos (+ laagpos 1))
			    )
				(if (= half 2)
					(if (= vwissel 0)
						(set! vwissel vrichting)
						(set! vwissel 0)
					)
				)
				(if (= half 1)
					(if (= hwissel 0)
						(set! hwissel hrichting)
						(set! hwissel 0)
					)
				)
			    (gimp-image-add-layer beeld werklaag nwpos)
			    (gimp-drawable-transform-2d-default werklaag 0 0 1 1 0 hwissel vwissel FALSE 0)
			    (set! lagenteller (- lagenteller 1))
			  )
		  )
		  (gimp-image-set-active-layer beeld baslaag)
		  (set! lagenteller diepte)
		  (while (> lagenteller 0)
			  (gimp-image-merge-down beeld baslaag 0)
			  (set! baslaag (car (gimp-image-get-active-layer beeld)))
			  (set! lagenteller (- lagenteller 1))
		  )
		;(gimp-image-set-active-layer beeld orglaag)
		(gimp-displays-flush)
		(gimp-image-undo-group-end beeld)
	  )  
	  )
	)

(define (black-gold-metal-mesh image 
                               layer
                               type
							   size
							   keep-selection-in
							   conserve
							   )
							   
	(gimp-image-undo-group-start image)						  

 (let* (
            (width (car (gimp-image-width image)))
			(height (car (gimp-image-height image)))
			(original-width width)
			(original-height height)
			(area (* 1000 1000))
			(alpha (car (gimp-drawable-has-alpha layer)))
		    (sel (car (gimp-selection-is-empty image)))
		    (layer-name (cond ((defined? 'gimp-image-get-item-position) (car (gimp-item-get-name layer)))
		            (else (car (gimp-drawable-get-name layer)))))
			(active-gradient (car (gimp-context-get-gradient)))
		    (active-fg (car (gimp-context-get-foreground)))
		    (active-bg (car (gimp-context-get-background)))
		    (keep-selection keep-selection-in)
			(selection-channel 0)
			(ver 2.8)
			(bkg-layer 0)
			(bkg-copy 0)
			(grid-layer 0)
			(pattern-layer 0)
			(bump-channel 0)
			(shadow-layer 0)
			(background 0)
			(type (cond ((= type 0) 1)
            ((= type 1) 0)
            (type)))
        )
	(cond ((not (defined? 'gimp-image-get-item-position)) (set! ver 2.6))) ;define the gimp version
	
	(gimp-context-push)
    (gimp-context-set-paint-method "gimp-paintbrush")
	(cond ((defined? 'gimp-context-set-dynamics) (gimp-context-set-dynamics "Dynamics Off")))
    (gimp-context-set-foreground '(0 0 0))
	(gimp-context-set-background '(255 255 255))

	(if (= alpha FALSE) (gimp-layer-add-alpha layer))
	
;;;;check that a selection was made if not make one	
	(if (= sel TRUE) (set! keep-selection FALSE))
	(if (= sel TRUE) (begin
	(cond ((= ver 2.8) (gimp-image-select-item image 2 layer)) 
	(else (gimp-selection-layer-alpha layer))
	) ;endcond
	)
	)
	
;;;;create selection-channel (gimp-selection-load selection-channel)	
	(set! selection-channel (car (gimp-selection-save image)))
	(cond ((= ver 2.8) (gimp-item-set-name selection-channel "selection-channel"))
	(else (gimp-drawable-set-name selection-channel "selection-channel"))
    ) ;endcond
	(gimp-image-set-active-layer image layer)
	(cond((= sel FALSE) (gimp-edit-clear layer))
	(else (gimp-selection-none image))
    ) ;endcond
	
;;;;begin the script	
	(set! bkg-layer (car (gimp-layer-new image width height RGBA-IMAGE "Background" 100 NORMAL-MODE)))
	(include-layer image bkg-layer layer 1)	;stack 0=above 1=below
	(plug-in-plasma 1 image bkg-layer 0 1.0)
	(set! bkg-copy (car (gimp-layer-copy bkg-layer TRUE)))
	(include-layer image bkg-copy bkg-layer 0)	;stack 0=above 1=below
	(gimp-layer-set-mode bkg-copy SCREEN-MODE)
	
	(set! grid-layer (car (gimp-layer-new image width height RGBA-IMAGE "Grid" 100 NORMAL-MODE)))
	(include-layer image grid-layer bkg-copy 0)	;stack 0=above 1=below
	(plug-in-grid 1 image grid-layer 1 2 0 "Black" 255 1 2 0 "Black" 255 1 2 0 "Black" 255)
	
	(set! pattern-layer (car (gimp-layer-new image width height RGBA-IMAGE "Pattern" 100 NORMAL-MODE)))
	(include-layer image pattern-layer grid-layer 0)	;stack 0=above 1=below
	(gimp-context-swap-colors)
	(gimp-drawable-fill pattern-layer FOREGROUND-FILL)
	(plug-in-mosaic 1 image 
	                pattern-layer ;drawable 
	                size ;tile-size 
					1 ;tile-height 
					(/ size 4) ;tile-spacing 
					1 ;tile-neatness 
					FALSE ;tile-allow-split 
					135 ;light-dir 
					0 ;color-variation 
					TRUE ;antialiasing 
					FALSE ;color-averaging 
					type ;tile-type Tile geometry { SQUARES (0), HEXAGONS (1), OCTAGONS (2), TRIANGLES (3) }
					0 ;tile-surface 
					0) ;Grout color (black/white or fore/background) { BW (0), FG-BG (1) }
	
	(cond ((= ver 2.8) (gimp-image-select-item image 2 pattern-layer)) 
	(else (gimp-selection-layer-alpha pattern-layer))
	) ;endcond
	(gimp-edit-clear pattern-layer)
	(gimp-selection-invert image)
	
;;;;create bump-channel (gimp-selection-load bump-channel)	
	(set! bump-channel (car (gimp-selection-save image)))
	(cond ((= ver 2.8) (gimp-item-set-name bump-channel "bump-channel"))
	(else (gimp-drawable-set-name bump-channel "bump-channel"))
    ) ;endcond
	(plug-in-gauss-rle2 RUN-NONINTERACTIVE image bump-channel 15 15)
	(gimp-image-set-active-layer image pattern-layer)	
	(gimp-context-set-background '(128 128 128))	
	(gimp-edit-blend pattern-layer FG-BG-RGB-MODE NORMAL-MODE GRADIENT-LINEAR 100 0 REPEAT-NONE FALSE FALSE 3 0.2 TRUE (/ width 2) 0 (/ width 2) height)
	(gimp-selection-none image)
	(plug-in-bump-map RUN-NONINTERACTIVE image pattern-layer bump-channel 135 45 32 0 0 0 0 TRUE FALSE 0)
	(plug-in-alienmap2 1 image pattern-layer 1 0 1 0 1 0 0 TRUE TRUE TRUE)
	
	(script-fu-drop-shadow image pattern-layer 4 4 15 '(0 0 0) 80 FALSE)
	(set! shadow-layer (car(find-layer-by-name image "Drop Shadow")))
	(gimp-layer-resize-to-image-size shadow-layer)	
	
;;;;finish the script	
	(if (= conserve FALSE) (begin
	(set! background (car (gimp-image-merge-down image pattern-layer EXPAND-AS-NECESSARY)))
	(set! background (car (gimp-image-merge-down image background EXPAND-AS-NECESSARY)))
	(set! background (car (gimp-image-merge-down image background EXPAND-AS-NECESSARY)))
	(set! background (car (gimp-image-merge-down image background EXPAND-AS-NECESSARY)))
	(cond ((= ver 2.8) (gimp-item-set-name background (string-append "Metal-Mesh" "\nBackground")))
	(else (gimp-drawable-set-name background (string-append "Metal-Mesh" "\nBackground")))
    ) ;endcond
	(set! layer (car (gimp-image-merge-down image layer EXPAND-AS-NECESSARY)))
	(cond ((= ver 2.8) (gimp-item-set-name layer (string-append layer-name "\nMetal-Mesh")))
	(else (gimp-drawable-set-name layer (string-append layer-name "\nMetal-Mesh")))
    ) ;endcond
	)) ;endif
   (if (= keep-selection TRUE) (gimp-selection-load selection-channel))
   (if (= conserve FALSE) (gimp-image-remove-channel image selection-channel))
   (if (= conserve FALSE) (gimp-image-remove-channel image bump-channel))

	(gimp-displays-flush)
	(gimp-image-undo-group-end image)
	(gimp-context-pop)

 )
)	